#!/usr/local/bin/python3
#  a simple CW dashboard maker for key IVS metrics
#  R.Clements 01-2021
#
import os
import json
from json import JSONEncoder
import code
import time
from time import gmtime, strftime
import datetime
from datetime import datetime
import subprocess 
import re
#####
myversion = "03"
#####
# ------------------------------------------------------------------------------------
## Definitions of various metrics: 
## ---------------------------------------------------------------------------
## Individual metrics dictionaries follow.
## NetworkIn widget dictionary template

def MakeDefaultWidget_LDT() :
	IVS_Channel_widget_d = {
            "type": "metric",
            "x": 0,
            "y": 0,
            "width": 18,
            "height": 6,
            "properties": {
                "metrics": [
                    [ "AWS/IVS", "LiveDeliveredTime", "Channel", "4aJxxxxx", { "label": "LiveDeliveredTime" } ]
                ],
                "view": "timeSeries",
                "stacked": "false",
                "region": "us-east-1",
                "stat": "Average",
                "period": 60,
                "yAxis": {
                    "left": {
                        "min": 0,
                        "label": "Seconds ( 1m avg)",
                        "showUnits": "false"
                    }
                },
                "title": "Live Delivered Time Metrics for Channel: MyChannel"
            }
        }
	return(IVS_Channel_widget_d)
##---------------------------------------------
	
def MakeDefaultWidget_ConcurrentStreams() :
	IVS_Channel_widget_d = {
            "type": "metric",
            "x": 0,
            "y": 0,
            "width": 18,
            "height": 6,
            "properties": {
                "metrics": [
                    [ "AWS/IVS", "ConcurrentStreams", "Channel", "mychannel", { "label": "ConcurrentStreams" } ]
                ],
                "view": "timeSeries",
                "stacked": "false",
                "region": "us-east-1",
                "stat": "Average",
                "period": 60,
                "yAxis": {
                    "left": {
                        "min": 0,
                        "label": "Seconds ( 1m avg)",
                        "showUnits": "false"
                    }
                },
                "title": "ConcurrentStreams for Channel: MyChannel"
            }
        }
	return(IVS_Channel_widget_d)
	
##-----------------------------------------------------
def MakeDefaultWidget_ConcurrentViews() :
	IVS_Channel_widget_d = {
            "type": "metric",
            "x": 0,
            "y": 0,
            "width": 18,
            "height": 6,
            "properties": {
                "metrics": [
                    [ "AWS/IVS", "ConcurrentViews", "Channel", "mychannel", { "label": "ConcurrentViews" } ]
                ],
                "view": "timeSeries",
                "stacked": "false",
                "region": "us-east-1",
                "stat": "Average",
                "period": 60,
                "yAxis": {
                    "left": {
                        "min": 0,
                        "label": "Seconds ( 1m avg)",
                        "showUnits": "false"
                    }
                },
                "title": "ConcurrentStreams for Channel: MyChannel"
            }
        }
	return(IVS_Channel_widget_d)
#
#---------------------------------------------------------------------------------------------
def GetCLIResult(mycmd):
        try:
                result = subprocess.run(mycmd, stdout=subprocess.PIPE)
                DecodedReply = json.loads(result.stdout.decode('utf-8'))
                return(DecodedReply)
        except:
                print("Could not get valid CLI resonse to cmd: ",mycmd)
                #code.interact(local=globals())
                exit()
#---------------------------------------------------------------------------------------------
# compute timestamps for files
tnow = datetime.now()
tstamp = tnow.strftime("%m%d%y_%H%M%S")
#---------------------------------------------------------------------------------------------

# Get IVS channel information for this acct
print("\n\n")
## aws cloudwatch list-metrics --namespace AWS/IVS
cmd = ['aws','ivs','list-channels']
try:
		CLIReply = GetCLIResult(cmd)
		 #print("DecodedReply has heys", CLIReply.keys())
except:
		print("!! No keys found in CLIReply for list-channels. Exiting .")
		exit()

CHLIST = CLIReply

# ------------------------------------------------------------------------------------
# Set region and compare later
myregion = "us-east-1"
CHcount = len(CHLIST['channels'])
if CHcount == 0:
	print("Couldnt find any IVS channels for this account in default AWS region.  Exiting ! ")
	exit()
# ------------------------------------------------------------------------------------
quote='"'
print("\n","========== WELCOME TO THE DASHBOARD MAKER v",myversion," ============")
print("   This script makes a Dash for: IVS Channel Monitoring","\n\n")

# ------------------------------------------------------------------------------------
## Iterate through all the discovered channels, adding a widget to the list
WidgetList=[]
for counter in range (0, CHcount):
	chName = ""
	chName = (CHLIST['channels'][counter]['name'])
	chName = chName.ljust(32, ' ')
	chArn = (CHLIST['channels'][counter]['arn'])
	#
	# extract channel ID from arn
	chanID = re.sub(r'^.*/',r'',chArn)
	print("\n","--> Found Channel ",chanID," with name ",chName)
	#-----------------------------------
	# Now check the region again
	arnstring=str(chArn)
	arnlist=arnstring.split(":",9)
	if (myregion != arnlist[3]) :
		myregion = arnlist[3]
		print("-----> Setting region to ",myregion," but you can change it later if needed.")
	#-----------------------------------
	#Now build the widget for this channel:
	try:
			TW1 = MakeDefaultWidget_LDT()
			TW1['properties']['metrics'][0][3] = chanID
			TW1['properties']['region']= myregion
			TW1['properties']['title'] ="Live Delivered Time Metrics for Channel: "+ chName
	except:
			print("!! Unable to add row 0 the widgets for Ch ",chanID, "...continuing..." )	
			#code.interact(local=globals())
	print("--> adding LDT widget")
##========================================	
	try:
			TW2 = MakeDefaultWidget_ConcurrentStreams()
			TW2['properties']['metrics'][0][3] = chanID
			TW2['properties']['region']= myregion
			TW2['properties']['title'] ="Concurrent Streams for Channel: "+ chName
	except:
			print("!! Unable to add row 0 the widgets for Ch ",chanID, "...continuing..." )	
			#code.interact(local=globals())
	print("--> adding ConcurrentStreams widget")
##========================================	
	try:
			TW3 = MakeDefaultWidget_ConcurrentViews()
			TW3['properties']['metrics'][0][3] = chanID
			TW3['properties']['region']= myregion
			TW3['properties']['title'] ="Concurrent Views for Channel: "+ chName
	except:
			print("!! Unable to add row 0 the widgets for Ch ",chanID, "...continuing..." )	
			#code.interact(local=globals())
	print("--> adding ConcurrentStreams widget")
	
	
	
	#---------------------------------------------
	try:
		TWL=['','','']
		TWL[0] = TW1
		TWL[1] = TW2
		TWL[2] = TW3  #  turn the temp dict into 1st item of a list
		WidgetList = WidgetList + TWL  # concatenate the latest widget to the full list
	except:
		print("!! -- > caught error during append temp widgets to list...exiting.")
		code.interact(local=globals())
		exit()

# ------------------------------------------------------------------------------------

## now we assemble the complete json 

FileOpen = '{"widgets": ['
FileClose = ']}'
## 
##  Start Assembling full JSON as a list of strings 
WidgetStringsList=[]
#
#
FullJSON = FileOpen
for counter in range (0, len(WidgetList)) :
	S = str(WidgetList[counter])
	S = S.replace('(','')
	S = S.replace(')','')
	S = S.replace("'",'"')
	S = S.replace('"false"','false')
	S = S.replace('"true"','true')
	S+=","
	FullJSON+=S
##
##remove the comma after final widget and finish the JSON
FullJSON = FullJSON[:-1]
FullJSON+=FileClose
##
print("")
dashfile = "dashjson_" + tstamp
f = open(dashfile, 'w')
f.write(FullJSON) 
f.close()
#
tyellow="\033[1;33;40m"
tnormal="\033[0;37;40m"
print(tnormal,"\n")


print("----> Dash JSON exported to local file '", dashfile, "' for future reference." )    
##
### now create the dashboard 
dashname = "IVS_Channel_Stats_" + tstamp
dashcmd = "aws cloudwatch put-dashboard --dashboard-name "+ dashname +" --dashboard-body file://" + dashfile
print("\n","================================================")
print(" Attempting to create new dashboard ",dashname, "in your CW Console. You can rename the dash via the Console.")
dashresult = os.popen(dashcmd).read()
print("\n\n"," Cloudwatch upload result was: (empty [ ] means no errors) ","\n",dashresult)
#code.interact(local=globals())
#print("Try running cmd: ", dashcmd)
print(" Remember to set the time range for your new Dashboard (upper right corner of CW Console).")

exit()
