#
#  a simple CW dashboard maker for key Elemental MediaConvert Status
#  R.Clements 04-2022
#  version 08
#  eliminate dependency on local flat files with cli-response-to-json function call
myversion = "04"
#
import os
import json
from json import JSONEncoder
import code
import time
from time import gmtime, strftime
import datetime
from datetime import datetime
import subprocess

## ------------------------------------------------------------------------------
## Tracks key metrics for each EMC Queue in customer account.
## ------------------------------------------------------------------------------

## Individual metrics dictionaries follow.
## First widget dictionary template:
def MakeDefaultWidget() :
	EMC_Status_widget_d =  {
            "type": "metric",
            "x": 0,
            "y": 0,
            "width": 12,
            "height": 3,
            "properties": {
                "metrics": [
                    [ "AWS/MediaConvert", "JobsCompletedCount", "Queue", "default-queue-arn", { "label": "Completed Jobs_Default Queue" } ],
                    [ ".", "JobsErroredCount", ".", ".", { "label": "Failed Jobs_Default Queue" } ],
                    [ ".", "JobsCanceledCount", ".", ".", { "label": "Cancelled Jobs_Default Queue" } ],
                ],
                "view": "singleValue",
                "region": "us-west-2",
                "stat": "Sum",
                "period": 86400,
                "title": "MediaConvert Jobs Status for Queue XX, Last 24H"
            }
        }
	return(EMC_Status_widget_d)
##-------------------------------------------------------------------------------------------------------
### template for lines 2-N
L1=[ ".", "JobsCompletedCount", "Queue", ".", { "label": "Completed Jobs_nextQ" } ]
L2=[ ".", "JobsErroredCount", "Queue", ".", { "label": "Failed Jobs_nextQ" } ]
L3=[ ".", "JobsCanceledCount", "Queue", ".", { "label": "Cancelled Jobs_nextQ" } ]


FinalLine=[ ".", "Errors", "Operation", "CreateJob" ]
#---------------------------------------------------------------------------------------------------------
def GetCLIResult(mycmd):
        try:
                result = subprocess.run(mycmd, stdout=subprocess.PIPE)
                DecodedReply = json.loads(result.stdout.decode('utf-8'))
                return(DecodedReply)
        except:
                print("L60: Could not get valid CLI resonse to cmd: ",mycmd)
                code.interact(local=globals())
                exit()
#-----------------------------------------------------------------------------------------------
print("\n\n")
print("\n","============= WELCOME TO THE DASHBOARD MAKER v",myversion," ===============")
print("   This script makes a Dash for:  MediaConvert Status Monitoring","\n\n")

#--------------------------------------------------------------------------------------
# compute timestamps for files+dashes
tnow = datetime.now()
tstamp = tnow.strftime("%m%d%y_%H%M%S")
#--------------------------------------------------------------------------------------
#  Get the default endpoint for this account
try:
	cmd1 = "aws mediaconvert describe-endpoints |grep Url|"
	cmd2 = cmd1 + "awk -F'\"' '{print $4}'"
	
	RES=(os.popen(cmd2).read())
	MyEndpoint = (RES.strip())
	#print("Using endpoint ",MyEndpoint)
except:
	print("L82: Could not get account specific enpoint. Sorry?  ...Exiting.")
	exit()
	
#---------------------------------------------------------------------------------
#Set region and compare later
currentregion = (MyEndpoint.split(".")[2])
print(" AWS Region defaulted to " + currentregion +" with endpoint:python3 EMC_status_dash_v03.py ",MyEndpoint,"\n Use this Region? [y]es, [c]hange : ")
rcheck = input(' ')
if  rcheck == "y" or rcheck =="Y" :
	myregion = currentregion
elif rcheck =="C" or rcheck == "c":
	cmd = ['/usr/local/bin/aws','--no-paginate','ec2','describe-regions']
	CLIReply = GetCLIResult(cmd)
	#code.interact(local=globals())
	regionlist=[]
	for Z in range (0, len(CLIReply['Regions'])):
		regionlist.append(CLIReply['Regions'][Z]['RegionName'])
	
	print("\n Current regions are: \n")
	for Z in range (0,len(regionlist)):
			print("   ",Z,"  ",regionlist[Z])
		
	ans=input("--> Enter number for new AWS region: ")
	## check the entered region
	myregion = regionlist[int(ans)]
	## reset the endpoint URL too
	cmd1 = "aws " +"--region "+ myregion + " mediaconvert describe-endpoints |grep Url|" + "awk -F'\"' '{print $4}'"
	RES=(os.popen(cmd1).read())
	MyEndpoint = (RES.strip())
print("\n--> Proceeding with AWS Region set to:",myregion, " and enpoint of: ",MyEndpoint)	
#code.interact(local=globals())
#--------------------------------------------------------------------------------------
# Get EMC Queue information for this acct from specified endpoint
print("\n\n")
cmd = ['aws','--region', myregion, '--endpoint', MyEndpoint, '--no-paginate','mediaconvert','list-queues']
try:
		CLIReply = GetCLIResult(cmd)
		#print("DecodedReply has heys", CLIReply.keys())
		QLIST = CLIReply
		Qcount = len(QLIST['Queues'])
except:
		print("!! Error getting response from list-queues call in this region.... Exiting .")
		exit()


#code.interact(local=globals())

	
#-------------------------------------------------------------------------------------------------------
TotalMetrics=0
MetricsList=[]
## Loop through all the Queues
for Qctr in range (0, Qcount):
## loop for all channels in this account:
	#print("\n\n\n","==>L117- The MetricsList json is now:", MetricsList, "\n\n")
	QName =  QLIST['Queues'][Qctr]['Name']
	print("----> Found Queue #",(Qctr+1)," named:",QName)
	TW = MakeDefaultWidget()
	# special case for first line of each metric due to CW formatting
	# this is cumbersome but not all widgets have common field names

	print("...")
	## set ARN and labels for the three lines that need them
	if Qctr > 0: 
		#print('.')
		tstamp2 = tnow.strftime("%m%d%y_%H%M%S")
	try:
		QName =  QLIST['Queues'][Qctr]['Name']
		#code.interact(local=globals())
		## metric line 1
		TW['properties']['metrics'][0][3] = QLIST['Queues'][Qctr]['Arn']
		TW['properties']['metrics'][0][4]['label'] = "Completed Jobs_"+ QName
		## metric line 2
		TW['properties']['metrics'][1][3] = QLIST['Queues'][Qctr]['Arn']
		TW['properties']['metrics'][1][4]['label'] = "Failed Jobs_"+ QName
		## metric line 3
		TW['properties']['metrics'][2][3] = QLIST['Queues'][Qctr]['Arn']
		TW['properties']['metrics'][2][4]['label'] = "Cancelled Jobs_"+ QName
		TW['properties']['title'] = "MediaConvert Jobs Status for Queue: "+ QName +" [Last 24H]"
		TW['properties']['region'] = myregion
		#
		TWL=['']
		TWL[0] =  TW
		
		MetricsList = MetricsList + TWL
		TotalMetrics+=1
		
	except:
		print("!!--> Caught error during populate temp widget line...please report this issue to the author. Exiting.")
		exit()	
		#code.interact(local=globals())

#----------------------------------------------------------------------------------------------------------------------		
## now we assemble the complete json 

FileOpen = '{"widgets": ['
FileClose = ']}'
print("\n\n","---> Assembling widget JSON...")
#code.interact(local=globals())
Assembled = FileOpen
for Z in range (0, len(MetricsList)) :
	Assembled = Assembled + str(MetricsList[Z]) + ','
#
# print("\n\n","Assembled is now:", Assembled)
# fix parens in string for json conversion
Assembled = Assembled.replace('(','')
Assembled = Assembled.replace(')','')

# fix more string to json formatting issues: quotes, booleans
Assembled = Assembled.replace("'",'"')
Assembled = Assembled.replace('"false"','false')
Assembled = Assembled.replace('"true"','true')
#
#remove final widget comma if present
if Assembled[(len(Assembled)-1)] == "," :
	Assembled = Assembled[:-1]
#	
Assembled = Assembled + FileClose
#print("---> The full assembled json is: ",Assembled)
#
print("")
dashfile = "dashjson_" + tstamp
f = open(dashfile, 'w')
f.write(Assembled) 
f.close()
tyellow="\033[1;33;40m"
tnormal="\033[0;37;40m"
print(tnormal,"\n")
#----------------------------------------------------------------------------------------------------------------------		
print("\n","==========================================================================")
print("\n"," - COMPLETE! -")
print("\n -> Dash JSON exported to local file '",dashfile, "' for future reference.","\n" )    
###############
### now create the dashboard 
tyellow="\033[1;33;40m"
tnormal="\033[0;37;40m"
dashname = "MediaConvert_Status_Dash_" + tstamp
dashcmd = "aws cloudwatch put-dashboard --dashboard-name "+ dashname +" --dashboard-body file://" + dashfile
print(" Attempting to create new dashboard ",tyellow, dashname, tnormal, "in your CW Console. You can rename the dash via the Console.")
dashresult = os.popen(dashcmd).read()
print("\n\n"," Cloudwatch upload result was: (empty [ ] means no errors) ","\n",dashresult)
print(" Remember to set the time range for your new Dashboard (upper right corner of CW Console).\n\n")
#print("Try running cmd: ",dashcmd)
#code.interact(local=globals())
exit()
