#!/usr/local/bin/python3
#  a simple CW dashboard maker for AWS Elemental service metrics
#  R.Clements 04-2021
#  this version supports selection of included channels by 
#  specific Tag with key = 'ProjectName' or by ch#
#
##------------------------------------------------------------------
import os
import json
from json import JSONEncoder
import code
import time
from time import gmtime, strftime
import datetime
from datetime import datetime
import subprocess 
import copy
##------------------------------------------------------------------
myversion = "07"
##
##------------------------------------------------------------------
tnow = datetime.now()
tstamp = tnow.strftime("%m%d%y_%H%M%S")   ## compute timestamps for files
#-----------------------------------------------------------------------------------------
def GetCLIResult(mycmd):
        try:
                result = subprocess.run(mycmd, stdout=subprocess.PIPE)
                DecodedReply = json.loads(result.stdout.decode('utf-8'))
                return(DecodedReply)
        except:
                print("Could not get valid CLI resonse to cmd: ",mycmd)
                #code.interact(local=globals())
                exit()
#-------------------------------------------------------------------------------------
quote='"'
print("\n","========== WELCOME TO THE EASY DASHBOARD MAKER for MediaPackage v",myversion," ============")
print("   This script makes a Dash for: MediaPackage Channel Monitoring","\n\n")
print("\n","\n",)
Answers = ['a','t','s']
RunMode = input("--> This script can include [a]ll channels, [t]agged channels, or a [s]ingle channel. Please choose a,t,s ? (any other char quits) " )
if RunMode not in Answers:
	print("----> exiting.") 
	exit()
	
##--------------------------------------------------------------------------------------
## Poll for all channels before we parse down the List
AllCannelsJSON_d = {'Channels': [{}]}  ## empty dictionaries for all channels and target channels
TargetChannelsJSON_d= {'Channels': []}
##
cmd = ['/usr/local/bin/aws','--no-paginate','mediapackage','list-channels']
try:
	AllCannelsJSON_d = GetCLIResult(cmd)
	## AllCannelsJSON is the list of all the channels we found. We will cull this list as needed
except:
	print("!! No valid reply from list-channels call. Sorry!!  --> Exiting .")
	exit()

ChCount = len(AllCannelsJSON_d['Channels'])
if (ChCount == 0 ):
	print("---> No Channels found for this Account in this region; exiting.")
	exit()	
		
##--------------------------------------------------------------------------------------
if RunMode == 't' :
	MyProjectName = input(" -> Please enter the value of the ProjectName to use when selecting Channels for this Dashboard: " )
	if (MyProjectName == ""): 
		print("  --> This option requires a value for the Tag called ProjectName. Exiting.")
		exit()
	
	for ChIndex in range (0, ChCount):  ## now we iterate all the chx matching tags
		ChPointer = AllCannelsJSON_d['Channels'][ChIndex]
		if ('Tags' in ChPointer) and ( 'ProjectName' in ChPointer['Tags']) and (ChPointer['Tags']['ProjectName'] == MyProjectName):
			print("----> Found matching channel ",ChPointer['Id'])
			TLL = len(TargetChannelsJSON_d['Channels'])
			#code.interact(local=globals())
			TargetChannelsJSON_d['Channels'].append(ChPointer)
			
		else:
			print(" ...")
	
	#code.interact(local=globals())
##------------------------------------------------------------------------------------
if RunMode == 's' :
	# display all channels and prompt for desired channel ID
	
	print("\n\n","  This account has the following channels defined:")	
	print("  ","DESC:","                               ","CH-ID:")
	for ChIndex in range (0, ChCount):  ## now we list all the chx
		
		try:
			#code.interact(local=globals())
			chanId = (AllCannelsJSON_d['Channels'][ChIndex]['Id'])
			if ('Description' in AllCannelsJSON_d['Channels'][ChIndex] ) : 
				chDesc = (AllCannelsJSON_d['Channels'][ChIndex]['Description'])
	
			else:
				chDesc = " "
		except:
			print("Could not retrieve all channel information :(  Exiting now. ")	
		
		
		chDesc = chDesc.ljust(32, ' ')
		chanID = (AllCannelsJSON_d['Channels'][ChIndex]['Id'])
		print("  ",chDesc, "    ",chanID)
	
	print("\n")
	SingleCh = input("--> Specify (copy+paste) the Channel ID to use: ")
	
	for ChIndex in range (0, ChCount):  ## now we iterate all the chx matching by ID
		ChPointer = AllCannelsJSON_d['Channels'][ChIndex]
		if (SingleCh == ChPointer['Id'] ):
			print("----> Adding single channel",ChPointer['Id'])
			#code.interact(local=globals())
			TargetChannelsJSON_d['Channels'].append(ChPointer)
			break
		else:
			print(" ...")
	
	#code.interact(local=globals())
##------------------------------------------------------------------------------------
if RunMode == 'a' :
	TargetChannelsJSON_d = copy.deepcopy(AllCannelsJSON_d)
	#code.interact(local=globals())
##------------------------------------------------------------------------------------
print("\n\n","---> Processing", len(TargetChannelsJSON_d['Channels'])," Channel(s) for new Dash...","\n")
##
##========================================================================================
## ==   WORK AGAINST TARGET LIST DCT FROM HERE ONWARD  ==
##========================================================================================
##
##----- Metrics Dictionaries ----------------------------------------
EMP_IngressResponseTime_widget_d = {
            "height": 6,
            "width": 18,
            "y": 0,
            "x": 0,
            "type": "metric",
            "properties": {
                "metrics": [
                    [ "AWS/MediaPackage", "IngressResponseTime", "Channel", "ChannelName"]
                ],
                "view": "timeSeries",
                "stacked": "false",
                "region": "us-west-2",
                "stat": "Average",
                "period": 60,
                "title": "Ingress Response Times - All Channels",
                "legend": {
                    "position": "right"
                },
                "yAxis": {
                    "left": {
                        "min": 0,
                        "label": "msec",
                        "showUnits": "false"
                    },
                    "right": {
                        "min": 0
                    }
                }
            }
}
#
##-------------------------------------------------------------------------------------------------------
##
EMP_IngressBytes_widget_d = {
            "height": 6,
            "width": 18,
            "y": 6,
            "x": 0,
            "type": "metric",
            "properties": {
                "metrics": [
                    [ "AWS/MediaPackage", "IngressBytes", "Channel", "ChannelName" ]
                ],
                "view": "timeSeries",
                "stacked": "false",
                "region": "us-west-2",
                "stat": "Average",
                "period": 60,
                "title": "Ingress Rate (Bytes) - All Channels",
                "legend": {
                    "position": "right"
                },
                "yAxis": {
                    "left": {
                        "min": 0,
                        "label": "Bytes",
                        "showUnits": "false"
                    },
                    "right": {
                        "min": 0
                    }
                }
            }
}
##-------------------------------------------------------------------------------------------------------
EMP_EgressResponseTime_widget_d = {
            "height": 6,
            "width": 18,
            "y": 12,
            "x": 0,
            "type": "metric",
            "properties": {
                "metrics": [
                    [ "AWS/MediaPackage", "EgressResponseTime", "Channel", "ChannelName" ]
                ],
                "view": "timeSeries",
                "stacked": "false",
                "region": "us-west-2",
                "stat": "Average",
                "period": 60,
                "title": "Egress Response Times - All Channels",
                "yAxis": {
                    "left": {
                        "min": 0,
                        "label": "msec",
                        "showUnits": "false"
                    },
                    "right": {
                        "min": 0
                    }
                },
                "legend": {
                    "position": "right"
                }
            }
        }
##-------------------------------------------------------------------------------------------------------
##
EMP_EgressRequestCount_widget_d = {
            "height": 6,
            "width": 9,
            "y": 18,
            "x": 0,
            "type": "metric",
            "properties": {
                "metrics": [
                    [ "AWS/MediaPackage", "EgressRequestCount", "Channel", "ChannelName", "StatusCodeRange", "2xx", { "label": "2xx Successful" } ],
                    [ "...", "4xx", { "label": "4xx Errors" } ],
                    [ "...", "5xx", { "label": "5xx Errors" } ]
                ],
                "view": "timeSeries",
                "stacked": "true",
                "region": "us-west-2",
                "stat": "Average",
                "period": 60,
                "title": "Egress Request Counts for Channel ",
                "yAxis": {
                    "left": {
                        "min": 0,
                        "label": "Status Codes",
                        "showUnits": "false"
                    },
                    "right": {
                        "min": 0
                    }
                },
                "setPeriodToTimeRange": "true",
                "legend": {
                    "position": "bottom"
                }
            }
 }

#-----------------------------------------------------------------------------------------------------
## Aggregate all the metrics dictionaries into a list of dictionaries
global AllChannelsWidgetsList
global PerChannelWidgets
AllChannelsWidgetsList = [EMP_IngressBytes_widget_d, EMP_IngressResponseTime_widget_d,EMP_EgressResponseTime_widget_d]
PerChannelWidgets=['']
#-------------------------------------------------------------------------------------------------------
# compute timestamps for files
tnow = datetime.now()
tstamp = tnow.strftime("%m%d%y_%H%M%S")


# ------------------------------------------------------------------------------------
# Set region and compare later
myregion = "us-east-1"
ChCount = len(TargetChannelsJSON_d['Channels'])
# ------------------------------------------------------------------------------------
quote='"'
print("\n","========== WELCOME TO THE DASHBOARD MAKER v",myversion," ============")
print("   This script makes a Dash for:  MediaPackage Channel Monitoring","\n\n")

# ------------------------------------------------------------------------------------
## Iterate through all the discovered channels
# set a boolean for need RTP widget
#
TotalChannelsProcessed = -1
#
for ChIndex in range (0, ChCount):
	chName = ''
	thislabel = ''
	MetricTemplateLine = [ "...", "SecondChannelName" ]
	#code.interact(local=globals())

	try:
		chanId = (TargetChannelsJSON_d['Channels'][ChIndex]['Id'])
		if ('Description' in TargetChannelsJSON_d['Channels'][ChIndex] ) : 
			chDesc = (TargetChannelsJSON_d['Channels'][ChIndex]['Description'])
	
		else:
			chDesc = " "
	except:
		print("Could not retrieve all channel information!  Exiting now. ")	
		
		
	chName = chanId.ljust(32, ' ')
	thislabel = "CH:" + chName
	print("\n","--> Scanning Channel ",chanId)
	cmd = ['/usr/local/bin/aws','--no-paginate','mediapackage','describe-channel','--id', chanId]
	try:
		CLIReply = GetCLIResult(cmd)
        
	except:
		print("!! No keys found in CLIReply for cmd: ",cmd)
		print("!! Exiting.")
		exit()

	ThisChanneljson = CLIReply
	
	# Now extract the region 
	arnstring=str(ThisChanneljson['Arn'])
	arnlist=arnstring.split(":",9)
	if (myregion != arnlist[3]) :
		myregion = arnlist[3]
		print("-----> Setting region to ",myregion," but you can change it later if needed.")
		
#################################################################################
## compute the per-channel Widgets    	
	chanId = (TargetChannelsJSON_d['Channels'][ChIndex]['Id'])
	PerChannelWidgets.append('')
	FOO=copy.deepcopy(EMP_EgressRequestCount_widget_d)
	PerChannelWidgets[ChIndex] = FOO
	PerChannelWidgets[ChIndex]['properties']['metrics'][0][3] = chanId
	WT = (PerChannelWidgets[ChIndex]['properties']['title'] + chanId)
	PerChannelWidgets[ChIndex]['properties']['title'] = WT
	#code.interact(local=globals())
##################################################################################		

	if (ChIndex == 0):
		# special case for first line of each metric due to param counts
		
		counter = 0
		for counter in range ( 0, len(AllChannelsWidgetsList) ) :
			try:
				AllChannelsWidgetsList[counter]['properties']['metrics'][0][3] = chanId
				#AllChannelsWidgetsList[counter]['properties']['metrics'][0][6]['label'] = thislabel
				AllChannelsWidgetsList[counter]['properties']['region']= myregion	
			except:
				print("unable to add row 0 the widgets for Ch ",chanId, "...continuing..." )

#########################################################################################	
	else:
		## we are in rows 2-N, so add a line to each metric in the all-channels-charts (not the 2xx4xx charts): 
		print("\n","Adding subsequent row to each widget for channel ",chanId)
		#for counter in range (0,len(AllChannelsWidgetsList)) :
			#AllChannelsWidgetsList[counter]['properties']['metrics'].append(MetricTemplateLine)
		##
		## now set the values		
		for counter in range (0,len(AllChannelsWidgetsList)) :
			try:
				NextLine = [ "...", chanId ]
				AllChannelsWidgetsList[counter]['properties']['metrics'].append(NextLine)
			
			except:
				print("unable to add a row to the widgets for Ch ",chanId, "...continuing..." )


		print("")
		
		## now do the per-channel widget 		
		
# ------------------------------------------------------------------------------------

## now we assemble the complete json 

FileOpen = '{"widgets": ['
FileClose = ']}'
## 
##  Start Assembling full JSON as a list of strings 
WidgetStringsList=[]
#
#
FullJSON = FileOpen
for counter in range (0, len(AllChannelsWidgetsList)) :
	S = str(AllChannelsWidgetsList[counter])
	S = S.replace('(','')
	S = S.replace(')','')
	S = S.replace("'",'"')
	S = S.replace('"false"','false')
	S = S.replace('"true"','true')
	S+=","
	FullJSON+=S

for counter in range (0, len(PerChannelWidgets)) :
	S = str(PerChannelWidgets[counter])
	S = S.replace('(','')
	S = S.replace(')','')
	S = S.replace("'",'"')
	S = S.replace('"false"','false')
	S = S.replace('"true"','true')
	S+=","
	FullJSON+=S
##
##remove the commas after final widget and finish the JSON
FullJSON = FullJSON[:-2]
FullJSON+=FileClose
##
print("")
dashfile = "MediaPackage_dashjson_" + tstamp
f = open(dashfile, 'w')
f.write(FullJSON) 
f.close()
#
tyellow="\033[1;33;40m"
tnormal="\033[0;37;40m"
print(tnormal,"\n")

print("----> Dash JSON exported to local file '", dashfile, "' for future reference." )    
##
### now create the dashboard 
tyellow="\033[1;33;40m"
tnormal="\033[0;37;40m"
dashname = "MediaPackage_Channels_Dash_" + tstamp
dashcmd = "aws cloudwatch put-dashboard --dashboard-name "+ dashname +" --dashboard-body file://" + dashfile
print("\n","================================================")
dashresult = os.popen(dashcmd).read()
print("\n\n"," Cloudwatch upload result was: (empty [ ] means no errors) ","\n",dashresult)
#code.interact(local=globals())
#print("Try running cmd: ", dashcmd)
print(" Remember to set the time range for your new Dashboard (upper right corner of CW Console).")

exit()
