#!/usr/local/bin/python3
#  a simple CW dashboard maker for key Elemental Mediastore metrics
#  NOT official AWS product in any way!  Just a helper script.
#  R.Clements 03-2021
#
import os
import json
from json import JSONEncoder
import code
import time
from time import gmtime, strftime
import datetime
from datetime import datetime
import subprocess 
import copy
##
myversion = "03"
# ------------------------------------------------------------------------------------
# Definition dictionaries of various metrics follow:
#
### Added DROPPED PACKETS - maximum widget?
EMS_ContainerMetrics_widget_d = {
            "type": "metric",
            "x": 0,
            "y": 0,
            "width": 24,
            "height": 6,
            "properties": {
    "metrics": [
        [ "AWS/MediaStore", "RequestCount", "ContainerName", "MyContainer", "RequestType", "PutRequests" ],
        [ ".", "4xxErrorCount", ".", ".", ".", "GetRequests" ],
        [ ".", "RequestCount", ".", ".", ".", "." ],
        [ ".", "BytesUploaded", ".", ".", { "yAxis": "right" } ],
        [ ".", "BytesDownloaded", ".", ".", { "yAxis": "right" } ]
    ],
    "view": "timeSeries",
    "stacked": "false",
    "region": "us-west-2",
    "stat": "Average",
    "period": 60,
    "title": "Container Metrics:",
    "yAxis": {
        "left": {
            "min": 0,
            "label": "Counts"
        },
        "right": {
            "min": 0,
            "label": "Bytes"
        }
    }
 }
}
##------------------------------------------------------------------------------------------------------  

#-----------------------------------------------------------------------------------------------------
## Aggregate all the metrics dictionaries into a list of dictionaries we will output
WidgetsList = []
#-----------------------------------------------------------------------------------------------------
def GetCLIResult(mycmd):
        try:
                result = subprocess.run(mycmd, stdout=subprocess.PIPE)
                DecodedReply = json.loads(result.stdout.decode('utf-8'))
                return(DecodedReply)
        except:
                print("Could not get valid CLI resonse to cmd: ",mycmd)
                #code.interact(local=globals())
                exit()
#-------------------------------------------------------------------------------------------------------
# compute timestamps for files
tnow = datetime.now()
tstamp = tnow.strftime("%m%d%y_%H%M%S")
#-------------------------------------------------------------------------------------------------------
quote='"'
print("\n","========== WELCOME TO THE EASY DASHBOARD MAKER v",myversion," ============")
print("   This script makes a Dash for: MediaStore Container Metrics Monitoring","\n\n")
print("\n","\n",)
Answers = ['a','t','s']
RunMode = input("--> This script can include [a]ll containers, [t]agged containers, or a [s]ingle container. Please choose a,t,s ? (any other char quits) " )
if RunMode not in Answers:
	print("----> exiting.") 
	exit()
	
	##--------------------------------------------------------------------------------------
## Poll for all channels before we parse down the List
AllConsJSON_d = {'Containers': [{}]}  ## empty dictionaries for all channels and target containers
containerLIST= {'Containers': []}
##

cmd = ['/usr/local/bin/aws','--no-paginate','mediastore','list-containers']
try:
	AllConsJSON_d = GetCLIResult(cmd)
	#print("\n\n"," Retrieved AllConsJSON_d  as :", AllConsJSON_d)
	## AllCannelsJSON is the list of all the containers/channels we found. We will cull this list as needed
except:
	print("!! No valid reply from list-containers call. Sorry!!  --> Exiting .")
	exit()

ChCount = len(AllConsJSON_d['Containers'])
if (ChCount == 0 ):
	print("---> No containers found for this Account in this region; exiting.")
	exit()	
		
##--------------------------------------------------------------------------------------
if RunMode == 't' :
	MyProjectName = input(" -> Please enter the value of the ProjectName to use when selecting containers for this Dashboard: " )
	if (MyProjectName == ""):   ## default value = 'myproject'
		print("  --> This option requires a value for the Tag called ProjectName. Exiting.")
		exit()
	
	#code.interact(local=globals())
	for ChIndex in range (0, ChCount):  ## now we iterate all the chx matching tags
		containerarn = AllConsJSON_d['Containers'][ChIndex]['ARN']
		
		cmd = ['aws','mediastore','list-tags-for-resource', '--resource',containerarn]
		containertags = GetCLIResult(cmd)
		print("\n\n","-->Retrieved tags: ",containertags, "from ",AllConsJSON_d['Containers'][ChIndex]['Name'])
		ChPointer = AllConsJSON_d['Containers'][ChIndex]
		#code.interact(local=globals())
		if ('Tags' in containertags) and (containertags['Tags'][0]['Value'] == MyProjectName):
			print("----> Found matching container ",AllConsJSON_d['Containers'][ChIndex]['Name'])
			TLL = len(containerLIST['Containers'])
			
			containerLIST['Containers'].append(ChPointer)
			
		else:
			print(" ...")
	
	#code.interact(local=globals())
##------------------------------------------------------------------------------------
if RunMode == 's' :
	# display all channels and prompt for desired channel ID
	
	print("\n\n","  This account has the following Containers defined:")	
	print("  ","NAME","                                ","ARN")
	for ChIndex in range (0, ChCount):  ## now we list all the chx
		chName = (AllConsJSON_d['Containers'][ChIndex]['Name'])
		chName = chName.ljust(32, ' ')
		chanARN = (AllConsJSON_d['Containers'][ChIndex]['ARN'])
		print("  ",chName, "    ",chanARN)
	
	print("\n")
	SingleCh = input("--> Specify (copy+paste) the Name of the Container to use: ")
	
	for ChIndex in range (0, ChCount):  ## now we iterate all the chx matching by ID
		ChPointer = AllConsJSON_d['Containers'][ChIndex]
		if (SingleCh == ChPointer['Name'] ):
			print("----> Adding single container",ChPointer['Name'])
			#code.interact(local=globals())
			containerLIST['Containers'].append(ChPointer)
			break
		else:
			print(" ...")
	
	#code.interact(local=globals())
##------------------------------------------------------------------------------------
if RunMode == 'a' :
	containerLIST = copy.deepcopy(AllConsJSON_d)
	#code.interact(local=globals())
##------------------------------------------------------------------------------------
print("\n\n","---> Processing", len(containerLIST['Containers'])," Channel(s) for new Dash...","\n")
##	
##===============================================================================================
## ===  WORK ON TARGET dict 'containerLIST' FROM THIS POINT FORWARD  ===========================
##===============================================================================================


# ------------------------------------------------------------------------------------
# Set region and compare later
myregion = "us-east-1"
containerCount = len(containerLIST['Containers'])
MetricTemplateLine = [ "...", "thisarn"]
# ------------------------------------------------------------------------------------
quote='"'
containerRow = -1
print("\n","========== WELCOME TO THE DASHBOARD MAKER v",myversion," ============")
print("   This script makes a CloudWatch Dashboard for:  Mediastore container Monitoring","\n\n")


#
# ------------------------------------------------------------------------------------
## Iterate through all the discovered containers add them to each widget 
# 
#
WidgetCount=-1
for containerIndex in range (0, containerCount):
	WidgetCount=WidgetCount+1
	containerName = ""
	thiscontainerName = (containerLIST['Containers'][containerIndex]['Name'])
	thiscontainerArn = (containerLIST['Containers'][containerIndex]['ARN'])
	containerName = containerName.ljust(32, ' ')
	thiscontainertitle = "Container Metrics: " + thiscontainerName
##
	print("\n","--> Scanning container ",thiscontainerName)
	cmd = ['/usr/local/bin/aws','mediastore','describe-container','--container-name', thiscontainerName ]
	try:
		CLIReply = GetCLIResult(cmd)
        #print("DecodedReply has heys", CLIReply.keys())
	except:
		print("!! No container info found on query: ",thiscontainerName)
		print("!! Please report this to the author via AWS Support. ")
		print("!! Exiting.")
		exit()

	Thiscontainerjson = CLIReply
	
	arnstring=str(thiscontainerArn)
	arnlist=arnstring.split(":",9)
	if (myregion != arnlist[3]) :
		myregion = arnlist[3]
		print("-----> Setting region to ",myregion," but you can change it later if needed.")

	
##---------------------------------------------------------------------------------
##


	try:
		TW=copy.deepcopy(EMS_ContainerMetrics_widget_d)  ## clone the template widget dictionary
		TW['properties']['metrics'][0][3] = thiscontainerName
		TW['properties']['title']= thiscontainertitle
		TW['properties']['region']= myregion
		WidgetsList.append(TW)	
	except:
		print("unable to add a widget for container ",thiscontainerName, "...continuing..." )

	
	print(".")

# ------------------------------------------------------------------------------------

## now we assemble the complete json 

FileOpen = '{"widgets": ['
FileClose = ']}'
## 
##  Start Assembling full JSON as a list of strings 
WidgetStringsList=[]
#
FullJSON = FileOpen
for counter in range (0, len(WidgetsList)) :
	S = str(WidgetsList[counter])
	S = S.replace('(','')
	S = S.replace(')','')
	S = S.replace("'",'"')
	S = S.replace('"false"','false')
	S = S.replace('"true"','true')
	S+=","
	FullJSON+=S
##
##remove the comma after final widget and finish the JSON
FullJSON = FullJSON[:-1]
FullJSON+=FileClose
##
print("")
dashfile = "MediaStore_dashjson_" + tstamp
f = open(dashfile, 'w')
f.write(FullJSON) 
f.close()
#
tyellow="\033[1;33;40m"
tnormal="\033[0;37;40m"
print(tnormal,"\n")


print("----> Dash JSON exported to local file '", dashfile, "' for future reference." )    
##
### now create the dashboard 
dashname = "MediaStore_containers_Dash_" + tstamp
dashcmd = "aws cloudwatch put-dashboard --dashboard-name "+ dashname +" --dashboard-body file://" + dashfile
print("\n","================================================")
print(" Attempting to create new dashboard ",dashname, "in your CW Console. You can rename the dash via the Console.")
dashresult = os.popen(dashcmd).read()
print("\n\n"," Cloudwatch upload result was:      (empty [ ] means no errors) ","\n",dashresult)
#code.interact(local=globals())
#print("Try running cmd: ", dashcmd)
print(" Remember to set the time range for your new Dashboard (upper right corner of CW Console).")

exit()
