#!/usr/local/bin/python3
#  a simple CW dashboard maker for AWS Elemental service metrics
#  R.Clements 07-2023
#  this version supports selection of included channels by 
#  specific Tag with key = 'ProjectName' or by ch#
#
##------------------------------------------------------------------
import os
import json
from json import JSONEncoder
import code
import time
from time import gmtime, strftime
import datetime
from datetime import datetime
import subprocess 
import copy
##------------------------------------------------------------------
myversion = "06"
##
##------------------------------------------------------------------
tnow = datetime.now()
tstamp = tnow.strftime("%m%d%y_%H%M%S")   ## compute timestamps for files
#-----------------------------------------------------------------------------------------
def GetCLIResult(mycmd):
        try:
                result = subprocess.run(mycmd, stdout=subprocess.PIPE)
                DecodedReply = json.loads(result.stdout.decode('utf-8'))
                return(DecodedReply)
        except:
                print("!Exiting...Could not get valid CLI resonse to cmd: ",mycmd)
                exit()
#-------------------------------------------------------------------------------------
quote='"'
print("\n","========== WELCOME TO THE EASY DASHBOARD MAKER v",myversion," ============")
print("   This script makes a CloudWatchDashboard for: MediaTailor Channel Assembly Monitoring","\n\n")
print("\n","\n",)
Answers = ['a','t','s']
RunMode = input("--> This script can include [a]ll channels, [t]agged channels, or a [s]ingle channel. Please choose a,t,s ? (any other char quits) " )
if RunMode not in Answers:
	print("----> exiting.") 
	exit()
	
##--------------------------------------------------------------------------------------
## NOTE:  EMT has both Live Linear Channel Assembly "Channels"  and Live Stream Campaign Configurations - we get metrics for both
## Poll for all channels before we parse down the List
AllCannelsJSON_d = {}  ## Empty dictionaries for all channels and target channels
TargetChannelsJSON_d= {}

cmd = ['/usr/local/bin/aws','--no-paginate','mediatailor','list-channels']
try:
	AllCannelsJSON_d = GetCLIResult(cmd)
	#code.interact(local=globals())
	##  note the Dict name is 'Items' but we want it as 'Channels'
	AllCannelsJSON_d['Channels'] = AllCannelsJSON_d['Items']
	del AllCannelsJSON_d['Items']

	## AllCannelsJSON is the list of all the Channels we found. We will cull this list as needed
except:
	print("!! No valid reply from list-channels call. Sorry!!  --> Exiting .")
	exit()

ChCount = len(AllCannelsJSON_d['Channels'])
if (ChCount == 0 ):
	print("---> No Channels found for this Account in this region; exiting.")
	exit()	
	
##--------------------------------------------------------------------------------------
myregion="us-west-2"
arnstring=str(AllCannelsJSON_d['Channels'][0]['Arn'])
arnlist=arnstring.split(":",9)
if (myregion != arnlist[3]) :
		myregion = arnlist[3]
		print("-----> Setting region to ",myregion," but you can change it later if needed.")	
##--------------------------------------------------------------------------------------
if RunMode == 't' :
	MyProjectName = input(" -> Please enter the value of the ProjectName to use when selecting VOD Channels for this Dashboard: " )
	if (MyProjectName == ""): 
		print("  --> This option requires a value for the Tag called ProjectName. Exiting.")
		exit()
		
	TargetChannelsJSON_d={'Channels':['']}
	TLL=0
	for ChIndex in range (0, ChCount):  ## now we iterate all the chx matching tags
		ChPointer = AllCannelsJSON_d['Channels'][ChIndex]
		if ('Tags' in ChPointer) and ( 'ProjectName' in ChPointer['Tags']) and (ChPointer['Tags']['ProjectName'] == MyProjectName):
			print("----> Found matching channel ",ChPointer['ChannelName'])
			#code.interact(local=globals())
			if TLL > 0 :
				TargetChannelsJSON_d['Channels'].append('')
				
			TargetChannelsJSON_d['Channels'][TLL]=ChPointer
			TLL = TLL+1
			#code.interact(local=globals())
		else:
			print(" ...")
	
	#code.interact(local=globals())
##------------------------------------------------------------------------------------
if RunMode == 's' :
	# display all channels and prompt for desired channel ID
	
	print("\n\n","  This account has the following Channel Assembly Channels defined:","\n")	
	print("  ","CHL NAME")
	print("  ","--------")
	for ChIndex in range (0, ChCount):  ## now we list all the chx

		try:
			#code.interact(local=globals())
			chanName = (AllCannelsJSON_d['Channels'][ChIndex]['ChannelName'])
			print("  ",chanName)
		except:
			print("Could not retrieve all channel information, sorry :(  Exiting now. ")	
	
	print("\n")
	SingleCh = input("--> Specify (copy+paste) the Channel Name to use for this Dashboard: ")
	
	for ChIndex in range (0, ChCount):  ## now we iterate all the chx matching by ID
		ChPointer = AllCannelsJSON_d['Channels'][ChIndex]
		if (SingleCh == ChPointer['ChannelName'] ):
			print("----> Adding single channel",ChPointer['ChannelName'])
			#code.interact(local=globals())
			TargetChannelsJSON_d={'Channels':['']}
			TargetChannelsJSON_d['Channels'][0]=ChPointer
			
		else:
			print(" ...")
	
	#code.interact(local=globals())
##------------------------------------------------------------------------------------
if RunMode == 'a' :
	TargetChannelsJSON_d = copy.deepcopy(AllCannelsJSON_d)
	#code.interact(local=globals())
##------------------------------------------------------------------------------------	
##
## print("TargetChannelsJSON_d is: ", TargetChannelsJSON_d)	
#print("\n\n","---> Processing", len(TargetChannelsJSON_d['Channels'])," Channel(s) for new Dash...","\n")
##
##========================================================================================
## ==   WORK AGAINST TARGET LIST DCT FROM HERE ONWARD  ==
##========================================================================================
##
##----- Metrics Dictionaries ----------------------------------------
EMT_VODChannel_widget_d = {
            "height": 6,
            "width": 18,
            "y": 0,
            "x": 0,
            "type": "metric",
            "properties": {
			"metrics": [
        			[ "AWS/MediaTailor", "TotalTime", "Output", "index", "Channel", "ChannelName", { "period": 60, "region": "us-west-2" } ],
        			[ ".", "5xxErrorCount", ".", ".", ".", ".", { "period": 60, "region": "us-west-2" } ],
        			[ ".", "4xxErrorCount", ".", ".", ".", "." , { "period": 60, "region": "us-west-2" } ],
        			[ ".", "RequestCount", ".", ".", ".", "." , { "period": 60, "region": "us-west-2" } ]
    			],
			"view": "timeSeries",
			"stacked": "false",
			"region": "myregiontext",
			"period": 60,
			"stat": "Average",
			"title": "VOD CH: ",
			"liveData": "true",
			"legend": {
				"position": "right"
			},
			"yAxis": {
				"right": {
					"min": 0,
					"showUnits": "false"
				},
				"left": {
					"showUnits": "false",
					"min": 0
				}
			}
		}
}
#
##-------------------------------------------------------------------------------------------------------
#
##-----------------------------------------------------------------------------------------------------
## Aggregate all the metrics dictionaries into a list of dictionaries
global PerChannelWidgets
PerChannelWidgets=['']
#-------------------------------------------------------------------------------------------------------
# compute timestamps for files
tnow = datetime.now()
tstamp = tnow.strftime("%m%d%y_%H%M%S")

# ------------------------------------------------------------------------------------
# Set region and compare later
ChCount = len(TargetChannelsJSON_d['Channels'])
# ------------------------------------------------------------------------------------
quote='"'

# ------------------------------------------------------------------------------------
## Iterate through all the discovered channels
# set a boolean for need RTP widget
#
TotalChannelsProcessed = -1
#
for ChIndex in range (0, ChCount):
	chName = ''
	thistitle = ''
	try:
		chanName = (TargetChannelsJSON_d['Channels'][ChIndex]['ChannelName'])

	except:
		print("Could not retrieve all channel information!  Exiting now. ")	
			
	
	thistitle = "VOD CH:" + chName
	print("\n","--> Scanning Channel ",chanName)
	cmd = ['/usr/local/bin/aws','mediatailor','describe-channel','--channel-name', chanName]
	try:
		CLIReply = GetCLIResult(cmd)
        
	except:
		print("!! No keys found in CLIReply for cmd: ",cmd)
		print("!! Exiting.")
		exit()

	ThisChanneljson = CLIReply
	
	# Now extract the region 
	arnstring=str(ThisChanneljson['Arn'])
	arnlist=arnstring.split(":",9)
	if (myregion != arnlist[3]) :
		myregion = arnlist[3]
		print("-----> Setting region to discovered value of ",myregion," but you can change it later if needed.")
		
#################################################################################
## compute the per-channel Widgets    	
	chanName = (TargetChannelsJSON_d['Channels'][ChIndex]['ChannelName'])
	PerChannelWidgets.append('')
	FOO=copy.deepcopy(EMT_VODChannel_widget_d)
	PerChannelWidgets[ChIndex] = FOO
	PerChannelWidgets[ChIndex]['properties']['metrics'][0][5] = chanName
	WT = (PerChannelWidgets[ChIndex]['properties']['title'] + chanName)
	PerChannelWidgets[ChIndex]['properties']['title'] = WT
	PerChannelWidgets[ChIndex]['properties']['region'] = myregion
	#code.interact(local=globals())
##################################################################################		
	#print("\n\n", "just built widget", PerChannelWidgets[ChIndex])
	

	print("...")
		
		## now do the per-channel widget 		
		
# ------------------------------------------------------------------------------------

## now we assemble the complete json 

FileOpen = '{"widgets": ['
FileClose = ']}'
## 
##  Start Assembling full JSON as a list of strings 
WidgetStringsList=[]
#
#
FullJSON = FileOpen

for counter in range (0, len(PerChannelWidgets)) :
	S = str(PerChannelWidgets[counter])
	S = S.replace('(','')
	S = S.replace(')','')
	S = S.replace("'",'"')
	S = S.replace('"false"','false')
	S = S.replace('"true"','true')
	S+=","
	FullJSON+=S
##
##remove the commas after final widget and finish the JSON
FullJSON = FullJSON[:-2]
FullJSON+=FileClose
##
print("")
dashfile = "MediaTailor_VODCHL_dashjson_" + tstamp
f = open(dashfile, 'w')
f.write(FullJSON) 
f.close()
#
tyellow="\033[1;33;40m"
tnormal="\033[0;37;40m"
print(tnormal,"\n")


print(tnormal,"----> Dash JSON exported to local file '", tyellow, dashfile, tnormal,"' for future reference." )    
##
### now create the dashboard 

dashname = "MediaTailor_Channels_Dash_" + tstamp
dashcmd = "aws cloudwatch put-dashboard --dashboard-name "+ dashname +" --dashboard-body file://" + dashfile
print(tnormal,"\n","================================================")
print(" Attempting to create new dashboard ",tyellow, dashname, tnormal, "in your CW Console. You can rename the dash via the Console.")
dashresult = os.popen(dashcmd).read()
print("\n\n"," Cloudwatch upload result: (empty set [ ] means no errors) ","\n\n")
print(dashresult)
#code.interact(local=globals())
#print("Try running cmd: ", dashcmd)
print("")
print(tyellow," ** Remember: You can adjust the timespan and time intervals for your new Dashboard (upper right corner of CW Console).",tnormal)

exit()
