#!/usr/local/bin/python3
#  a simple CW dashboard maker for AWS Elemental service metrics
#  R.Clements 06-2021
#  
##------------------------------------------------------------------
import os
import json
from json import JSONEncoder
import code
import time
from time import gmtime, strftime
import datetime
from datetime import datetime
import subprocess 
import copy
##------------------------------------------------------------------
myversion = "03"
##
##------------------------------------------------------------------
tnow = datetime.now()
tstamp = tnow.strftime("%m%d%y_%H%M%S")   ## compute timestamps for files
#-----------------------------------------------------------------------------------------
#
def GetCLIResult(mycmd):
        try:
                result = subprocess.run(mycmd, stdout=subprocess.PIPE)
                DecodedReply = json.loads(result.stdout.decode('utf-8'))
                return(DecodedReply)
        except:
                print("!Exiting...Could not get valid CLI resonse to cmd: ",mycmd)
                exit()
#-------------------------------------------------------------------------------------
#
quote='"'
print("\n","========== WELCOME TO THE EASY DASHBOARD MAKER v",myversion," ============")
print("   This script makes a Dash for: MediaTailor SSAI Configuration Metrics Monitoring","\n\n")
print("\n","\n",)
Answers = ['a','t','s']
RunMode = input("--> This script can include [a]ll SSAI Configurations, [t]agged Configs, or a [s]ingle Configuration. Please choose a,t,s ? (any other char quits) " )
if RunMode not in Answers:
	print("----> exiting.") 
	exit()
	
##--------------------------------------------------------------------------------------
## NOTE:  EMT has both Live Linear Channel Assembly "Configs"  and Live Stream Configuration Configurations - we get metrics for both
## Poll for all Configs before we parse down the List
AllConfigsJSON_d = {}  ## Empty dictionaries for all Configs and target Configs
TargetConfigsJSON_d= {}

cmd = ['/usr/local/bin/aws','--no-paginate','mediatailor','list-playback-configurations']
try:
	AllConfigsJSON_d = GetCLIResult(cmd)
	#code.interact(local=globals())
	##  note the Dict name is 'Items' but we want it as 'Configs'
	AllConfigsJSON_d['Configs'] = AllConfigsJSON_d['Items']
	del AllConfigsJSON_d['Items']

	## AllConfigsJSON is the list of all the Configs we found. We will cull this list as needed.
except:
	print("!! No valid reply from list-playback-configurations call. Sorry!!  --> Exiting .")
	exit()

CfgCount = len(AllConfigsJSON_d['Configs'])
if (CfgCount == 0 ):
	print("---> No Configs found for this Account in this AWS Region; exiting.")
	exit()	
	
##--------------------------------------------------------------------------------------
myregion="us-west-2"
arnstring=str(AllConfigsJSON_d['Configs'][0]['PlaybackConfigurationArn'])
arnlist=arnstring.split(":",9)
if (myregion != arnlist[3]) :
		myregion = arnlist[3]
		print("-----> Setting region to ",myregion," but you can change it later if needed.")	
##--------------------------------------------------------------------------------------
if RunMode == 't' :
	MyProjectName = input(" -> Please enter the Value associated with the Tag key 'ProjectName' to use when selecting Configuration Configs for this Dashboard: " )
	if (MyProjectName == ""): 
		print("  --> This option requires a value for the Tag key called 'ProjectName'. Exiting.")
		exit()
		
	TargetConfigsJSON_d={'Configs':['']}
	TLL=0
	for CfgIndex in range (0, CfgCount):  ## now we iterate all the chx matching tags
		ChPointer = AllConfigsJSON_d['Configs'][CfgIndex]
		if ('Tags' in ChPointer) and ( 'ProjectName' in ChPointer['Tags']) and (ChPointer['Tags']['ProjectName'] == MyProjectName):
			print("----> Found matching channel ",ChPointer['Name'])
			#code.interact(local=globals())
			if TLL > 0 :
				TargetConfigsJSON_d['Configs'].append('')
				
			TargetConfigsJSON_d['Configs'][TLL]=ChPointer
			TLL = TLL+1
			#code.interact(local=globals())
		else:
			print(" ...")
	
##------------------------------------------------------------------------------------
if RunMode == 's' :
	# display all Configs and prompt for desired channel ID
	
	print("\n\n","  This account has the following SSAI Configuration Configs defined:","\n")	
	print("  ","CFG NAME")
	print("  ","--------")
	for CfgIndex in range (0, CfgCount):  ## now we list all the chx

		try:
			#code.interact(local=globals())
			ConfigName = (AllConfigsJSON_d['Configs'][CfgIndex]['Name'])
			print("  ",ConfigName)
		except:
			print("Could not retrieve all config information, sorry :(  Exiting now. ")	
			exit()
	
	print("\n")
	SingleCh = input("--> Specify (copy+paste) the Configuration Name to include in this Dashboard: ")
	
	for CfgIndex in range (0, CfgCount):  ## now we iterate all the chx matching by ID
		ChPointer = AllConfigsJSON_d['Configs'][CfgIndex]
		if (SingleCh == ChPointer['Name'] ):
			print("----> Adding single config",ChPointer['Name'])
			#code.interact(local=globals())
			TargetConfigsJSON_d={'Configs':['']}
			TargetConfigsJSON_d['Configs'][0]=ChPointer
			
		else:
			print(" ...")
	
	#code.interact(local=globals())
##------------------------------------------------------------------------------------
if RunMode == 'a' :
	TargetConfigsJSON_d = copy.deepcopy(AllConfigsJSON_d)
	#code.interact(local=globals())
##------------------------------------------------------------------------------------	
##
## print("TargetConfigsJSON_d is: ", TargetConfigsJSON_d)	
print("\n\n","---> Processing", len(TargetConfigsJSON_d['Configs'])," Channel(s) for new Dash...","\n")
##
##--------------------------------------------------------------------------------------
## ==   WORK AGAINST TARGET LIST DCT FROM HERE ONWARD  ==
##
##----- Metrics Dictionaries ----------------------------------------
EMT_CfgSummary_widget_d = {
            "type": "metric",
            "x": 0,
            "y": 0,
            "width": 12,
            "height": 6,
            "properties": {
                "metrics": [
                    [ "AWS/MediaTailor", "Requests", "ConfigurationName", "namehere" ],
                    [ "AWS/MediaTailor", "AdsBilled", "ConfigurationName", "namehere" ],
                    [ "AWS/MediaTailor", "AdDecisionServer.FillRate", "ConfigurationName", "namehere" ],
                    [ ".", "Origin.Timeouts", ".", "." ]
                ],
                "view": "timeSeries",
                "stacked": "false",
                "region": "us-west-2",
                "stat": "Average",
                "period": 60,
                "title": "SSAI Configuration Top Level Metrics",
                "yAxis": {
                    "left": {
                        "min": 0
                    }
                },
                "liveData": "true"
            }
}
#
##-------------------------------------------------------------------------------------------------------

EMT_FillRates_widget_d = {
            "type": "metric",
            "x": 0,
            "y": 0,
            "width": 12,
            "height": 6,
            "properties": {
                "metrics": [
                	[ "AWS/MediaTailor", "AdDecisionServer.FillRate", "ConfigurationName", "namehere" ],
                	[ "AWS/MediaTailor", "Avail.FillRate", "ConfigurationName", "namehere" ],
                	[ "AWS/MediaTailor", "Avail.ObservedFillRate","ConfigurationName", "namehere"],
                	[ ".", "SkippedReason.DurationExceeded", ".", "." ]
                ],
                "view": "timeSeries",
                "stacked": "false",
                "region": "us-west-2",
                "title": "Fill Rate Metrics",
                "yAxis": {
                    "left": {
                        "min": 0
                    }
                },
                "stat": "Average",
                "period": 60
            }
}
##-------------------------------------------------------------------------------------------------------
EMT_Errors_widget_d = {
            "type": "metric",
            "x": 0,
            "y": 0,
            "width": 12,
            "height": 6,
            "properties": {
                "metrics": [
                    [ "AWS/MediaTailor", "AdDecisionServer.Errors", "ConfigurationName", "namehere" ],
                    [ "AWS/MediaTailor", "AdDecisionServer.Timeouts", "ConfigurationName", "namehere" ],
                    [ "AWS/MediaTailor", "AdDecisionServer.Latency", "ConfigurationName", "namehere" ],
                    [ ".", "Origin.Errors", ".", "." ],
                    [ ".", "Origin.Timeouts", ".", "." ]
                ],
                "view": "timeSeries",
                "stacked": "false",
                "region": "us-west-2",
                "yAxis": {
                    "left": {
                        "min": 0
                    }
                },
                "stat": "Average",
                "period": 60,
                "title": "Error Metrics"
            }
}


##-----------------------------------------------------------------------------------------------------
## Aggregate all the metrics dictionaries into a list of dictionaries
global PerCfgWidgets
PerCfgWidgets=['']
#-------------------------------------------------------------------------------------------------------
# compute timestamps for files
tnow = datetime.now()
tstamp = tnow.strftime("%m%d%y_%H%M%S")

# ------------------------------------------------------------------------------------
# Set region and compare later
CfgCount = len(TargetConfigsJSON_d['Configs'])
# ------------------------------------------------------------------------------------
quote='"'

# ------------------------------------------------------------------------------------
## Iterate through all the discovered Configs
# set a boolean for need RTP widget
#
TotalConfigsProcessed = -1
WidetsAdded = -1
#
for CfgIndex in range (0, CfgCount):
	cfgName = ''
	thistitle = ''
	try:
		cfgName = (TargetConfigsJSON_d['Configs'][CfgIndex]['Name'])
	
	except:
		print("Could not retrieve needed configuration information!  Exiting now. ")	
			
	
	thistitle = "Ad Config:" + cfgName
	print("\n","--> Scanning Config ",cfgName)
	cmd = ['/usr/local/bin/aws','mediatailor','get-playback-configuration','--name', cfgName]
	try:
		CLIReply = GetCLIResult(cmd)
        
	except:
		print("--> !! No info found when getting playback configuration! ")
		print("--> !! Exiting.")
		exit()

	ThisChanneljson = CLIReply
	
	# Now extract the region 
	arnstring=str(ThisChanneljson['PlaybackConfigurationArn'])
	arnlist=arnstring.split(":",9)
	if (myregion != arnlist[3]) :
		myregion = arnlist[3]
		print("-----> Setting region to discovered value of ",myregion," but you can change it later if needed.")
		
##--------------------------------------------------------------------------------------
## compute the per-channel Widgets    	
	ConfigName = (TargetConfigsJSON_d['Configs'][CfgIndex]['Name'])
	### instantiate each widget dictionary for each config found
	PerCfgWidgets.append('')
	WidetsAdded = WidetsAdded+1
	##---- first widget ##-------------------------------------
	FOO=copy.deepcopy(EMT_CfgSummary_widget_d)
	PerCfgWidgets[WidetsAdded] = FOO
	PerCfgWidgets[WidetsAdded]['properties']['metrics'][0][3] = ConfigName
	PerCfgWidgets[WidetsAdded]['properties']['metrics'][1][3] = ConfigName
	PerCfgWidgets[WidetsAdded]['properties']['metrics'][2][3] = ConfigName
	PerCfgWidgets[WidetsAdded]['properties']['metrics'][3][3] = ConfigName
	WT = (PerCfgWidgets[WidetsAdded]['properties']['title'] + ": "+ConfigName)
	PerCfgWidgets[WidetsAdded]['properties']['title'] = WT
	PerCfgWidgets[WidetsAdded]['properties']['region'] = myregion

	##---- second widget ##-------------------------------------
	PerCfgWidgets.append('')
	WidetsAdded=WidetsAdded+1
	F2=copy.deepcopy(EMT_FillRates_widget_d)
	PerCfgWidgets[WidetsAdded] = F2
	PerCfgWidgets[WidetsAdded]['properties']['metrics'][0][3] = ConfigName
	PerCfgWidgets[WidetsAdded]['properties']['metrics'][1][3] = ConfigName
	PerCfgWidgets[WidetsAdded]['properties']['metrics'][2][3] = ConfigName
	PerCfgWidgets[WidetsAdded]['properties']['metrics'][3][3] = ConfigName
	WT = (PerCfgWidgets[WidetsAdded]['properties']['title'] + ": "+ConfigName)
	PerCfgWidgets[WidetsAdded]['properties']['title'] = WT
	PerCfgWidgets[WidetsAdded]['properties']['region'] = myregion

	##---- third widget ##-------------------------------------
	PerCfgWidgets.append('')
	WidetsAdded=WidetsAdded+1
	F3=copy.deepcopy(EMT_Errors_widget_d)
	PerCfgWidgets[WidetsAdded] = F3
	PerCfgWidgets[WidetsAdded]['properties']['metrics'][0][3] = ConfigName
	PerCfgWidgets[WidetsAdded]['properties']['metrics'][1][3] = ConfigName
	PerCfgWidgets[WidetsAdded]['properties']['metrics'][2][3] = ConfigName
	PerCfgWidgets[WidetsAdded]['properties']['metrics'][3][3] = ConfigName
	PerCfgWidgets[WidetsAdded]['properties']['metrics'][4][3] = ConfigName
	WT = (PerCfgWidgets[WidetsAdded]['properties']['title'] + ": "+ConfigName)
	PerCfgWidgets[WidetsAdded]['properties']['title'] = WT
	PerCfgWidgets[WidetsAdded]['properties']['region'] = myregion
	
	
##--------------------------------------------------------------------------------------
	print("...")
		
## END OF ITERATING LOOP
## now we assemble the complete json 

FileOpen = '{"widgets": ['
FileClose = ']}'
## 
##  Start Assembling full JSON as a list of strings 
WidgetStringsList=[]
#
#
FullJSON = FileOpen

for counter in range (0, len(PerCfgWidgets)) :
	S = str(PerCfgWidgets[counter])
	S = S.replace('(','')
	S = S.replace(')','')
	S = S.replace("'",'"')
	S = S.replace('"false"','false')
	S = S.replace('"true"','true')
	S+=","
	FullJSON+=S
##
##remove the commas after final widget and finish the JSON
FullJSON = FullJSON[:-2]
FullJSON+=FileClose
##
print("")
dashfile = "MediaTailor_Configurations_dashjson_" + tstamp
f = open(dashfile, 'w')
f.write(FullJSON) 
f.close()
#
tyellow="\033[1;33;40m"
tnormal="\033[0;37;40m"
print(tnormal,"\n")


print(tnormal,"----> Dash JSON exported to local file '", tyellow, dashfile, tnormal,"' for future reference." )    
##
### now create the dashboard 

dashname = "MediaTailor_SSAI_Dash_" + tstamp
dashcmd = "aws cloudwatch put-dashboard --dashboard-name "+ dashname +" --dashboard-body file://" + dashfile
print(tnormal,"\n","================================================","\n")
print(" Attempting to create new dashboard ",tyellow, dashname, tnormal, "in your CW Console. You can rename the dash via the Console.")
dashresult = os.popen(dashcmd).read()
print("\n\n"," Cloudwatch upload result was: (empty [ ] means no errors) ","\n\n",dashresult)
#code.interact(local=globals())
#print("Try running cmd: ", dashcmd)
print("\n")
print(tyellow," ** Remember: You can to adjust the granularity and time range for your new Dashboard (upper right corner of CW Console).")

exit()
