#
#  a simple CW dashboard maker for key Elemental MediaConvert Status
#  R.Clements 12-2020
#  version 08
#  eliminate dependency on local flat files with cli-response-to-json function call
myversion = "04"
#
import os
import json
from json import JSONEncoder
import code
import time
from time import gmtime, strftime
import datetime
from datetime import datetime
import subprocess
import operator
from operator import itemgetter

## ------------------------------------------------------------------------------
## Tracks key metrics for each EMC Queue in customer account.
## ------------------------------------------------------------------------------

## Individual metrics dictionaries follow.
## First widget dictionary template:
def MakeDefaultWidget() :
	Bkt_widget_d = {
            "type": "metric",
            "x": 0,
            "y": 6,
            "width": 12,
            "height": 3,
            "properties": {
                "metrics": [
                    [ "AWS/S3", "NumberOfObjects", "StorageType", "AllStorageTypes", "BucketName", "mybkt", { "label": "AllStorageTypes NumberOfObjects label" } ],
                    [ ".", "BucketSizeBytes", ".", "StandardStorage", ".", "." ],
                    [ "...", "GlacierStorage", ".", ".", { "label": "GlacierStorage BucketSizeBytes label" } ]
                ],
                "view": "singleValue",
                "stacked": "false",
                "region": "us-west-2",
                "title": "Bucket: XXX",
                "period": 604800,
                "stat": "Sum"
            }
        }
	return(Bkt_widget_d)

#---------------------------------------------------------------------------------------------------------
def GetCLIResult(mycmd):
        try:
                result = subprocess.run(mycmd, stdout=subprocess.PIPE)
                DecodedReply = json.loads(result.stdout.decode('utf-8'))
                return(DecodedReply)
        except:
                print("L60: Could not get valid CLI resonse to cmd: ",mycmd)
                #code.interact(local=globals())
                exit()
#-----------------------------------------------------------------------------------------------
print("\n\n")
print("==================== WELCOME TO THE DASHBOARD MAKER v",myversion," ===================")
print("  This script makes CloudWatch Dashes for: Stats on Each S3 Bucket on your account")
print("\n\n")
#--------------------------------------------------------------------------------------
# compute timestamps for files+dashes
tnow = datetime.now()
tstamp = tnow.strftime("%m%d%y_%H%M%S")
#--------------------------------------------------------------------------------------
#Set region and compare later
currentregion = "us-east-1"

cmd="aws configure list |grep region |awk -F' ' '{print $2}'"
R=(os.popen(cmd).read())
myregion = (R.strip())

#--------------------------------------------------------------------------------------
# Get S3 bucket information for this acct 
print("\n\n")
cmd = ['aws','s3api', 'list-buckets']
try:
		CLIReply = GetCLIResult(cmd)
		#print("DecodedReply has heys", CLIReply.keys())
		BKLIST = CLIReply
		Bcount = len(BKLIST['Buckets'])
except:
		print("!! L84: Error getting response from list-buckets call. Exiting .")
		exit()


#code.interact(local=globals())
	
#-----------------------------------------------------------------------------------------
TotalWidgets=0
WidgetsList=[]
bktnumber = 0
## WidgetsList reads [size,widget_dictionary]
## Loop through all the buckets & create a widget
for counter in range (0, Bcount):
	bktnumber +-1
	Bname =  BKLIST['Buckets'][counter]['Name']
	print("----> Found Bucket #",(counter+1)," named:",Bname)
	#
	# get the size of this bkt for sorting the list later - TIME CONSUMING!
	#cmd="aws s3 ls --summarize --recursive s3://"+Bname+" | tail -n 1 | awk -F' ' '{print $3}'"
	#R=(os.popen(cmd).read())
	#bktsize = (R.strip())
	#
	#Create a temp widget to populate and append to the widgets list
	TW = MakeDefaultWidget()
	#print("...getting size of Bucket ",Bname," \n")
	try:
		## metric line 1
		TW['properties']['title'] = " Bucket: "+Bname
		TW['properties']['region'] = myregion
		TW['properties']['metrics'][0][5] = Bname
		TWL=[]
		TWL.append([bktnumber,TW])
		## can later sort by bktnumber if desired
		WidgetsList = WidgetsList + TWL
		TotalWidgets+=1
		
	except:
		print("!!--> L126 - caught error during populate temp widget line...exiting.")
		#code.interact(local=globals())
		exit()	

#----------------------------------------------------------------------------------------------------------------------		
## now we assemble the complete json 

FileOpen = '{"widgets": ['
FileClose = ']}'
print("\n\n","---> Assembling widget JSON...")
#code.interact(local=globals())
getsizes = itemgetter(0)
sortedlist = sorted(WidgetsList, key=getsizes)
#
Assembled = FileOpen
for Z in range (0, len(WidgetsList)) :
	Assembled = Assembled + str(WidgetsList[Z][1]) + ','
#
# fix parens in string for json conversion
Assembled = Assembled.replace('(','')
Assembled = Assembled.replace(')','')

# fix more string to json formatting issues: quotes, booleans
Assembled = Assembled.replace("'",'"')
Assembled = Assembled.replace('"false"','false')
Assembled = Assembled.replace('"true"','true')
#
#remove final widget comma if present
if Assembled[(len(Assembled)-1)] == "," :
	Assembled = Assembled[:-1]
#	
Assembled = Assembled + FileClose
#print("---> The full assembled json is: ",Assembled)
#
print("")
dashfile = "dashjson_" + tstamp
f = open(dashfile, 'w')
f.write(Assembled) 
f.close()
#---------------------------------------------
#
tyellow="\033[1;33;40m"
tnormal="\033[0;37;40m"
print(tnormal,"\n")

#----------------------------------------------------------------------------------------------------------------------		
print("\n","==========================================================================")
print("\n"," - COMPLETE! -")
print("\n -> Dash JSON exported to local file '",dashfile, "' for future reference.","\n" )    
###############
### now create the dashboard 
dashname = "S3_Bucket_Stats_" + tstamp
dashURL = "https://"+ myregion +".console.aws.amazon.com/cloudwatch/home?region=" + myregion +"#dashboards:name=" + dashname +";start=P7D"
dashcmd = "aws cloudwatch put-dashboard --dashboard-name "+ dashname +" --dashboard-body file://" + dashfile
print("Attempting to create new dashboard ",tyellow, dashname, tnormal," In your CW Console. You can rename the Dash via the console.")
print (" Your new dashboard will be at",tyellow, dashURL, tnormal)
dashresult = os.popen(dashcmd).read()
print("\n\n"," Cloudwatch upload result was: (empty [ ] means no errors) ","\n",dashresult)
print(" Remember, you can adjust the time range for your new Dashboard (upper right corner of CW Console).\n\n")
#print("Try running cmd: ",dashcmd)
#code.interact(local=globals())
exit()
